/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "file.h"
#include <stdlib.h>
#include <stdio.h>

#ifdef HAVE_STRING_H
# include <string.h>
#else
# include <strings.h>
#endif /* HAVE_STRING_H */

struct File* File_init (const char* name)
{
  struct File* f = malloc (sizeof(struct File));
  f->name = malloc (strlen (name) + 1);
  strcpy (f->name, name);
  
  f->fp = fopen (f->name, "rb");
  f->wflag = 0;
  if (f->fp == 0)
  	f->size = 0;
  else {
  	long l;
  	File_seek_end (f);
  	l = File_tell (f);
  	f->size = (l>0)? l:0;
  	File_seek (f, 0);
  }
  return f;
}

void File_begin_read (const struct File* ff)
{
  struct File* f = (struct File*)ff;
  if (f->wflag) {
  	fclose (f->fp);
  	f->fp = fopen (f->name, "rb");
  	f->wflag = 0;
  }
  else
  	File_reset (f);
}

void File_append (const struct File *ff)
{
  struct File* f = (struct File*)ff;
  if (f->fp)
  	fclose (f->fp);
  f->fp = fopen (f->name, "rb+");
  File_seek_end (f);
  f->wflag = 1;
  f->size = 0;
}

void File_write_new (const struct File* ff)
{
  struct File* f = (struct File*)ff;
  if (f->fp)
  	fclose (f->fp);
  f->fp = fopen (f->name, "wb");
  f->wflag = 1;
  f->size = 0;
}

int File_ok (const struct File* f)
{
  return f->fp != 0;
}

int File_wflag (const struct File* f)
{
  return f->wflag;
}

int File_empty (const struct File* f)
{
  return f->size == 0;
}

const char* File_name (const struct File* f)
{
  return f->name;
}

void File_set_size (const struct File* f, long l)
{
  struct File* ff = (struct File*)f;
  ff->size = l;
}

void File_seek (const struct File* f, long pos)
{
  fseek (f->fp, pos, SEEK_SET);
}

void File_seek_end (const struct File* f)
{
  fseek (f->fp, 0, SEEK_END);
}

long File_tell (const struct File* f)
{
  return ftell (f->fp);
}

long File_size (const struct File* f)
{
  return f->size;
}

void File_reset (const struct File* f)
{
  fseek (f->fp, 0, SEEK_SET);
}

void File_close (const struct File* f)
{
  if (!f) return;
  free (f->name);
  free ((char*)f);
}
